/*
 * data.h
 *
 * application data
 *
 * 23/08/2017    GF    Created
 * 
 * 
 * MIT License
 * 
 * Copyright 2017 Graeme Foot (Kinetic Engineering Design Ltd.)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef DATA_H
#define	DATA_H


#include <stdint.h>

#include <rtai_types.h>
#include <asm/rtai_atomic.h>

#include "etherCATMaster.h"



#define EXIT_ERROR 10



/**
 * the global application data structure
 */
typedef struct _app_s
{
  volatile atomic_t     shutdown;                /**< a flag indicating the system should shut down */

  uint64_t              scanTimeNS;              /**< the amount of time per scan in nanoseconds */
  uint64_t              scanTimeMS;              /**< the amount of time per scan in milliseconds */
  double                scanTimeSec;             /**< the amount of time per scan in seconds */

  int32_t               rtTimeDiff;              /**< master time source time diff (ns) */
  int32_t               rtTimeDelta;             /**< master time source time diff delta (ns) */
  uint64_t              rtTimeStart;             /**< master time source time base start (ns) */
  uint64_t              rtTimeCurr;              /**< master time source time base current (ns) */
  int64_t               rtAdjBy;                 /**< master time source cycle adjustment value (ns) */

  uint64_t              nextWakeTime;            /**< next cycle wake time time (ns) */
  uint64_t              actualWakeTime;          /**< actual cycle wake time (ns) */
  uint64_t              overruns;                /**< the number of overruns that have occured */
  
  RT_TASK              *appTask;                 /**< RTAI task ref */
  etherCATMaster_s     *ecatMaster;              /**< EtherCAT master */ 
  
} app_s;


/** the global application data entry point
 */
extern app_s g_app;



/** get the rt time in ns for the current cpu (adjusted for the app time base)
 * 
 * \ret the time in ns
 */
uint64_t app_getTimeNS();



/** convert app time to rtai time in counts (via the app time base)
 */
RTIME app2rtaiTime(
        uint64_t in_appTime
        );


/** add to the app time base value
 * 
 * if the app clock is slower (ie the period takes longer) then the time base
 * value should be increased each period
 */
void app_addTimeBase(
        int64_t in_timeBase
        );



/**
 * log message to syslog
 */
void logMsg(const char *__format, ...);



/** wait for the next period
 */
void ecat_waitPeriod();


#endif	/* DATA_H */

